-- XAPP132 
--  
-- DLL Board Clock Mirror for Virtex chips Example
--
-- Requires external connection of CLK0_ext to CLKFB on
-- the board.
--

library ieee;
use ieee.std_logic_1164.all;
-- pragma translate_off
library unisim;
use unisim.vcomponents.all;
-- pragma translate_on

entity dll_mirror_2 is
    port (CLKIN      : in  std_logic;
          CLKFB      : in  std_logic;
          CLK0_ext   : out std_logic;
          CLK0_int   : out std_logic;
          NOT_LOCKED : out std_logic);
end dll_mirror_2;

architecture structural of dll_mirror_2 is

component IBUFG
-- synopsys translate_off
   generic(
      TimingChecksOn: Boolean := DefaultTimingChecksOn;
      InstancePath: STRING := "*";
      Xon: Boolean := DefaultXon;
      MsgOn: Boolean := DefaultMsgOn;
      tpd_I_O                        :	VitalDelayType01 := (0.100 ns, 0.100 ns);
      tipd_I                         :	VitalDelayType01 := (0.000 ns, 0.000 ns));

-- synopsys translate_on
   port(
      O                              :	out   STD_ULOGIC;
      I                              :	in    STD_ULOGIC);
end component;

component CLKDLL
-- synopsys translate_off
    generic ( TimingChecksOn : Boolean := DefaultTimingChecksOn;
              InstancePath : STRING := "*";
              Xon  : Boolean := DefaultXon;
             MsgOn : Boolean := DefaultMsgOn;

             tipd_CLKIN   : VitalDelayType01 := (0.000 ns, 0.000 ns);
             tipd_CLKFB   : VitalDelayType01 := (0.000 ns, 0.000 ns);
             tipd_RST     : VitalDelayType01 := (0.000 ns, 0.000 ns);

             tpd_CLKIN_LOCKED  : VitalDelayType01 := (0.100 ns, 0.100 ns);

             tperiod_CLKIN_POSEDGE     : VitalDelayType  := 0.010 ns;
             MAXPERCLKIN       : time := 100 ns;

             tpw_CLKIN_posedge : VitalDelayType  := 0.010 ns;
             tpw_CLKIN_negedge : VitalDelayType  := 0.010 ns;

             tpw_RST_posedge   : VitalDelayType  := 0.010 ns;

              DUTY_CYCLE_CORRECTION : Boolean := TRUE;
              CLKDV_DIVIDE : real := 2.0);

-- synopsys translate_on
    port ( CLKIN   : in  std_ulogic := '0';
           CLKFB   : in  std_ulogic := '0';
           RST     : in  std_ulogic := '0';
           CLK0    : out std_ulogic := '0';
           CLK90   : out std_ulogic := '0';
           CLK180  : out std_ulogic := '0';
           CLK270  : out std_ulogic := '0';
           CLK2X   : out std_ulogic := '0';
           CLKDV   : out std_ulogic := '0';
           LOCKED  : out std_ulogic := '0');
end component;
component BUFG
-- synopsys translate_off
   generic(
      TimingChecksOn: Boolean := DefaultTimingChecksOn;
      InstancePath: STRING := "*";
      Xon: Boolean := DefaultXon;
      MsgOn: Boolean := DefaultMsgOn;
      tpd_I_O                        :	VitalDelayType01 := (0.100 ns, 0.100 ns);
      tipd_I                         :	VitalDelayType01 := (0.000 ns, 0.000 ns));

-- synopsys translate_on
   port(
      O                              :	out   STD_ULOGIC;
      I                              :	in    STD_ULOGIC);
end component;
component OBUF
-- synopsys translate_off
   generic(
      TimingChecksOn: Boolean := DefaultTimingChecksOn;
      InstancePath: STRING := "*";
      Xon: Boolean := DefaultXon;
      MsgOn: Boolean := DefaultMsgOn;
      tpd_I_O                        :	VitalDelayType01 := (0.100 ns, 0.100 ns);
      tipd_I                         :	VitalDelayType01 := (0.000 ns, 0.000 ns));

-- synopsys translate_on
   port(
      O                              :	out   STD_ULOGIC;
      I                              :	in    STD_ULOGIC);
end component;


signal CLKIN_w, CLKFB_w, CLK0_int_dll, CLK0_int_g, CLK0_ext_dll : std_logic;
signal LOCKED_ext_dll, NOT_LOCKED_w : std_logic;
signal logic0 : std_logic;

begin 

logic0 <= '0';

clkpad   : IBUFG  port map (I=>CLKIN, O=>CLKIN_w);
clkfbpad : IBUFG  port map (I=>CLKFB, O=>CLKFB_w);

dllint : CLKDLL port map (CLKIN=>CLKIN_w, CLKFB=>CLK0_int_g, RST=>logic0,
                          CLK0=>CLK0_int_dll, CLK90=>open, CLK180=>open, CLK270=>open,
                          CLK2X=>open,    CLKDV=>open, LOCKED=>open);

dllext : CLKDLL port map (CLKIN=>CLKIN_w, CLKFB=>CLKFB_w, RST=>logic0,
                          CLK0=>CLK0_ext_dll, CLK90=>open, CLK180=>open, CLK270=>open,
                          CLK2X=>open,    CLKDV=>open, LOCKED=>LOCKED_ext_dll);

clkg      : BUFG   port map (I=>CLK0_int_dll, O=>CLK0_int_g);
clkextpad : OBUF   port map (I=>CLK0_ext_dll, O=>CLK0_ext);

CLK0_int <= CLK0_int_g;

NOT_LOCKED_w <= not LOCKED_ext_dll;

notlckpad : OBUF   port map (I=>NOT_LOCKED_w, O=>NOT_LOCKED);

end structural;
